@@ -14,7 +14,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; TODO Macro for generating different variants:
 ;;; :ucs-2le (little endian)    sap-ref-16le
@@ -347,7 +347,7 @@
                             (dpb (cref 3) (byte 8 24)
                                  (dpb (cref 2) (byte 8 16)
                                       (dpb (cref 1) (byte 8 8) (cref 0))))))))
-          (if (< code sb!xc:char-code-limit)
+          (if (< code sb-xc:char-code-limit)
               (code-char code)
               (decoding-error array pos (+ pos bytes) :ucs-4le
                               'octet-decoding-error pos))))
@@ -366,13 +366,13 @@
                             (dpb (cref 0) (byte 8 24)
                                  (dpb (cref 1) (byte 8 16)
                                       (dpb (cref 2) (byte 8 8) (cref 3))))))))
-          (if (< code sb!xc:char-code-limit)
+          (if (< code sb-xc:char-code-limit)
               (code-char code)
               (decoding-error array pos (+ pos bytes) :ucs-4be
                               'octet-decoding-error pos)))))))
 
 (eval-when (:compile-toplevel)
-  (sb!xc:proclaim '(muffle-conditions compiler-note)))
+  (sb-xc:proclaim '(muffle-conditions compiler-note)))
 (instantiate-octets-definition define-simple-get-ucs4-character)
 
 (defmacro define-ucs-4->string (accessor type)
@@ -424,7 +424,7 @@
   (setf (sap-ref-32le sap tail) bits)
   4
   (let ((code (sap-ref-32le sap head)))
-    (if (< code sb!xc:char-code-limit)
+    (if (< code sb-xc:char-code-limit)
         (code-char code)
         (return-from decode-break-reason 4)))
   ucs-4le->string-aref
@@ -436,7 +436,7 @@
   (setf (sap-ref-32be sap tail) bits)
   4
   (let ((code (sap-ref-32be sap head)))
-    (if (< code sb!xc:char-code-limit)
+    (if (< code sb-xc:char-code-limit)
         (code-char code)
         (return-from decode-break-reason 4)))
   ucs-4be->string-aref
