@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;;; miscellaneous non-primitive predicates
 
@@ -87,12 +87,11 @@
 ;;; All the primitive type predicate wrappers share a parallel form..
 (macrolet ((def-type-predicate-wrapper (pred)
              (let* ((name (symbol-name pred))
-                    (package (package-name (symbol-package pred)))
                     (stem (string-left-trim "%" (string-right-trim "P-" name)))
                     (article (if (position (schar name 0) "AEIOU") "an" "a")))
-               (aver (not (string= package "SB!XC")))
                `(defun ,pred (object)
-                  ,@(unless (eql (mismatch package "SB!") 3)
+                  ;; Document the standardized predicates and not the internal ones.
+                  ,@(when (eql (sb-xc:symbol-package pred) *cl-package*)
                       (list (format nil
                                     "Return true if OBJECT is ~A ~A, and NIL otherwise."
                                     article
@@ -142,6 +141,7 @@
   (def-type-predicate-wrapper short-float-p)
   (def-type-predicate-wrapper single-float-p)
   #!+sb-simd-pack (def-type-predicate-wrapper simd-pack-p)
+  #!+sb-simd-pack-256 (def-type-predicate-wrapper simd-pack-256-p)
   (def-type-predicate-wrapper %instancep)
   (def-type-predicate-wrapper funcallable-instance-p)
   (def-type-predicate-wrapper symbolp)
@@ -166,8 +166,8 @@
                   ,@(map 'list
                          (lambda (saetp)
                            `(def-type-predicate-wrapper
-                                ,(symbolicate (sb!vm:saetp-primitive-type-name saetp) "-P")))
-                         sb!vm:*specialized-array-element-type-properties*))))
+                                ,(symbolicate (sb-vm:saetp-primitive-type-name saetp) "-P")))
+                         sb-vm:*specialized-array-element-type-properties*))))
     (saetp-defs))
   ;; Other array types
   (def-type-predicate-wrapper simple-array-p)
@@ -194,7 +194,7 @@
   (declare (optimize (speed 3) (safety 0)))
   #!+(and compact-instance-header x86-64)
   (values (%primitive layout-of x
-                      (load-time-value sb!kernel::**built-in-class-codes** t)))
+                      (load-time-value sb-kernel::**built-in-class-codes** t)))
   #!-(and compact-instance-header x86-64)
   (cond ((%instancep x) (%instance-layout x))
         ((funcallable-instance-p x) (%funcallable-instance-layout x))
@@ -204,7 +204,7 @@
         (t
          ;; Note that WIDETAG-OF is slightly suboptimal here and could be
          ;; improved - we've already ruled out some of the lowtags.
-         (svref (load-time-value sb!kernel::**built-in-class-codes** t)
+         (svref (load-time-value sb-kernel::**built-in-class-codes** t)
                 (widetag-of x)))))
 
 (declaim (inline classoid-of))
@@ -229,10 +229,10 @@
      (cond
        ((<= 0 object 1) 'bit)
        ((< object 0) 'fixnum)
-       (t '(integer 0 #.sb!xc:most-positive-fixnum))))
+       (t '(integer 0 #.sb-xc:most-positive-fixnum))))
     (integer
      (if (>= object 0)
-         '(integer #.(1+ sb!xc:most-positive-fixnum))
+         '(integer #.(1+ sb-xc:most-positive-fixnum))
          'bignum))
     (character
      (typecase object
@@ -246,21 +246,21 @@
     (symbol
      (cond ((eq object t) 'boolean)
            ((eq object nil) 'null)
-           ((eq (symbol-package object) *keyword-package*) 'keyword)
+           ((eq (sb-xc:symbol-package object) *keyword-package*) 'keyword)
            (t 'symbol)))
-    ((or array complex #!+sb-simd-pack simd-pack)
-     (let ((sb!kernel::*unparse-allow-negation* nil))
-       (declare (special sb!kernel::*unparse-allow-negation*)) ; forward ref
+    ((or array complex #!+sb-simd-pack simd-pack #!+sb-simd-pack-256 simd-pack-256)
+     (let ((sb-kernel::*unparse-allow-negation* nil))
+       (declare (special sb-kernel::*unparse-allow-negation*)) ; forward ref
        (type-specifier (ctype-of object))))
     (t
      (let* ((classoid (classoid-of object))
             (name (classoid-name classoid)))
        (if (%instancep object)
            (case name
-             (sb!alien-internals:alien-value
+             (sb-alien-internals:alien-value
               `(alien
-                ,(sb!alien-internals:unparse-alien-type
-                  (sb!alien-internals:alien-value-type object))))
+                ,(sb-alien-internals:unparse-alien-type
+                  (sb-alien-internals:alien-value-type object))))
              (t
               (let ((pname (classoid-proper-name classoid)))
                 (if (classoid-p pname)
@@ -381,8 +381,8 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
           ((consp x) (and (consp y)
                           (recurse (car x) (car y))
                           (recurse (cdr x) (cdr y))))
-          ((and (symbolp x) (not (symbol-package x)))
-           (and (symbolp y) (not (symbol-package y)) (string= x y)))
+          ((and (symbolp x) (not (sb-xc:symbol-package x)))
+           (and (symbolp y) (not (sb-xc:symbol-package y)) (string= x y)))
           (t
            (equal x y)))))
 
@@ -411,20 +411,20 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
                   `(let ((x-el (%instance-ref x i))
                          (y-el (%instance-ref y i)))
                      (or (eq x-el y-el) (equalp x-el y-el)))))
-         (if (eql (layout-bitmap layout-x) sb!kernel::+layout-all-tagged+)
-             (loop for i of-type index from sb!vm:instance-data-start
+         (if (eql (layout-bitmap layout-x) sb-kernel::+layout-all-tagged+)
+             (loop for i of-type index from sb-vm:instance-data-start
                    below (layout-length layout-x)
                    always (slot-ref-equalp))
              (let ((comparators (layout-equalp-tests layout-x)))
                (unless (= (length comparators)
-                          (- (layout-length layout-x) sb!vm:instance-data-start))
+                          (- (layout-length layout-x) sb-vm:instance-data-start))
                  (bug "EQUALP got incomplete instance layout"))
                ;; See remark at the source code for %TARGET-DEFSTRUCT
                ;; explaining how to use the vector of comparators.
-               (loop for i of-type index from sb!vm:instance-data-start
+               (loop for i of-type index from sb-vm:instance-data-start
                      below (layout-length layout-x)
                      for test = (data-vector-ref
-                                 comparators (- i sb!vm:instance-data-start))
+                                 comparators (- i sb-vm:instance-data-start))
                      always (cond ((eql test 0) (slot-ref-equalp))
                                   ((functionp test)
                                    (funcall test i x y))
