@@ -11,7 +11,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; utilities
 
@@ -125,6 +125,7 @@
      ((lvar-delayed-leaf lvar)
       (setf (ir2-lvar-kind info) :delayed))
      (t (let ((tn (make-normal-tn (ir2-lvar-primitive-type info))))
+          (setf (tn-type tn) (lvar-type lvar))
           (setf (ir2-lvar-locs info) (list tn))
           (when (lvar-dynamic-extent lvar)
             (setf (ir2-lvar-stack-pointer info)
@@ -246,7 +247,7 @@
 
 ;;; Annotate LVAR for a fixed, but arbitrary number of values, of the
 ;;; specified primitive TYPES.
-(defun annotate-fixed-values-lvar (lvar types)
+(defun annotate-fixed-values-lvar (lvar types &optional lvar-types)
   (declare (type lvar lvar) (list types))
   (let ((info (make-ir2-lvar nil)))
     (setf (ir2-lvar-locs info)
@@ -254,6 +255,10 @@
                 collect (if type
                             (make-normal-tn type)
                             (make-unused-tn))))
+    (when lvar-types
+      (loop for type in lvar-types
+            for tn in (ir2-lvar-locs info)
+            do (setf (tn-type tn) type)))
     (setf (lvar-info lvar) info)
     (when (lvar-dynamic-extent lvar)
       (aver (proper-list-of-length-p types 1))
@@ -288,7 +293,7 @@
   (let* ((lvar (return-result node))
          (fun (return-lambda node))
          (returns (tail-set-info (lambda-tail-set fun)))
-         (types (return-info-types returns)))
+         (types (return-info-primitive-types returns)))
     (if (eq (return-info-count returns) :unknown)
         (collect ((res *empty-type* values-type-union))
           (do-uses (use (return-result node))
@@ -305,8 +310,10 @@
               (if (eq kind :unknown)
                   (annotate-unknown-values-lvar lvar)
                   (annotate-fixed-values-lvar
-                   lvar (mapcar #'primitive-type types))))))
-        (annotate-fixed-values-lvar lvar types)))
+                   lvar (mapcar #'primitive-type types)
+                   types)))))
+        (annotate-fixed-values-lvar lvar types
+                                    (return-info-types returns))))
 
   (values))
 
@@ -318,7 +325,9 @@
   (declare (type mv-combination call))
   (setf (basic-combination-kind call) :local)
   (setf (node-tail-p call) nil)
-  (let ((args (basic-combination-args call)))
+  (let ((args (basic-combination-args call))
+        (vars (lambda-vars
+               (ref-leaf (lvar-use (basic-combination-fun call))))))
     (if (singleton-p args)
         (annotate-fixed-values-lvar
          (first args)
@@ -330,20 +339,27 @@
                       nil)
                      (t
                       (primitive-type (basic-var-type var)))))
-                 (lambda-vars
-                  (ref-leaf (lvar-use (basic-combination-fun call))))))
+                 vars)
+         (mapcar #'basic-var-type vars))
         (let ((types (mapcar (lambda (var)
-                               (primitive-type (basic-var-type var)))
-                             (lambda-vars
-                              (ref-leaf (lvar-use (basic-combination-fun call)))))))
+                               (cons (primitive-type (basic-var-type var))
+                                     (basic-var-type var)))
+                             vars)))
           (dolist (arg args)
-            (annotate-fixed-values-lvar
-             arg
-             (loop repeat (nth-value 1 (values-types
-                                        (lvar-derived-type arg)))
-                   collect (if types
-                               (pop types)
-                               *backend-t-primitive-type*)))))))
+            (collect ((lvar-types)
+                      (primitive-types))
+              (let ((n-values (nth-value 1 (values-types
+                                            (lvar-derived-type arg)))))
+                (loop repeat n-values
+                      for (prim-type . lvar-type) = (pop types)
+                      do
+                      (primitive-types (or prim-type
+                                           *backend-t-primitive-type*))
+                      (lvar-types (or lvar-type
+                                      *universal-type*)))
+                (annotate-fixed-values-lvar
+                 arg
+                 (primitive-types) (lvar-types))))))))
   (values))
 
 ;;; We force all the argument lvars to use the unknown values
@@ -489,7 +505,7 @@
          (flet ((type-p (value type)
                   (if (typep type '(cons (eql satisfies)))
                       (funcall (second type) value)
-                      (sb!xc:typep value type))))
+                      (sb-xc:typep value type))))
           (cond (lvar
                  (and (constant-lvar-p lvar)
                       (type-p (lvar-value lvar) (cdr restr))))
@@ -837,7 +853,7 @@
                     (name (lvar-fun-name (combination-fun call))))
                 (and (leaf-has-source-name-p funleaf)
                      (eq name (leaf-source-name funleaf))
-                     (not (sb!vm::static-fdefn-offset name))
+                     (not (sb-vm::static-fdefn-offset name))
                      (let ((info (basic-combination-fun-info call)))
                        (not (or (fun-info-ir2-convert info)
                                 (ir1-attributep (fun-info-attributes info)
@@ -879,6 +895,7 @@
 
 (defun ltn-annotate-casts (lvar)
   (declare (type lvar lvar))
+  (process-annotations lvar)
   (do-uses (node lvar)
     (when (cast-p node)
       (ltn-annotate-cast node))))
@@ -897,10 +914,11 @@
               (ctype (lvar-derived-type value)))
          (multiple-value-bind (types rest)
              (values-type-types ctype (specifier-type 'null))
-           (annotate-fixed-values-lvar
-            value
-            (mapcar #'primitive-type
-                    (adjust-list types count rest))))))))
+           (let ((types (adjust-list types count rest)))
+             (annotate-fixed-values-lvar
+              value
+              (mapcar #'primitive-type types)
+              types)))))))
   (values))
 
 
