@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 
 ;; If 'plausible-signed-imm32-operand-p' is true, use it; otherwise use a RIP-relative constant.
@@ -170,8 +170,14 @@
                   (:translate ,translate)
                   (:generator 1
                    ,@(or c/fixnum=>fixnum
-                         `((move r x)
-                           (inst ,op r (constantize (fixnumize y)))))))
+                         `((cond
+                             ,@(and (eq op 'sub)
+                                    `(((and (not (location= r x))
+                                            (typep (- (fixnumize y)) '(signed-byte 32)))
+                                       (inst lea r (ea (- (fixnumize y)) x)))))
+                             (t
+                              (move r x)
+                              (inst ,op r (constantize (fixnumize y)))))))))
                 (define-vop (,(symbolicate "FAST-" translate "/SIGNED=>SIGNED")
                              fast-signed-binop)
                   (:translate ,translate)
@@ -182,7 +188,14 @@
                   (:translate ,translate)
                   (:generator ,untagged-penalty
                    ,@(or c/signed=>signed
-                         `((move r x) (inst ,op r (constantize y))))))
+                         `((cond
+                             ,@(and (eq op 'sub)
+                                    `(((and (not (location= r x))
+                                            (typep (- y) '(signed-byte 32)))
+                                       (inst lea r (ea (- y) x)))))
+                             (t
+                              (move r x)
+                              (inst ,op r (constantize y))))))))
                 (define-vop (,(symbolicate "FAST-"
                                            translate
                                            "/UNSIGNED=>UNSIGNED")
@@ -197,7 +210,14 @@
                   (:translate ,translate)
                   (:generator ,untagged-penalty
                    ,@(or c/unsigned=>unsigned
-                         `((move r x) (inst ,op r (constantize y)))))))))
+                         `((cond
+                             ,@(and (eq op 'sub)
+                                    `(((and (not (location= r x))
+                                            (typep (- y) '(signed-byte 32)))
+                                       (inst lea r (ea (- y) x)))))
+                             (t
+                              (move r x)
+                              (inst ,op r (constantize y)))))))))))
 
   ;;(define-binop + 4 add)
   (define-binop - 4 sub)
@@ -217,7 +237,7 @@
   (define-binop logior 2 or
     :c/unsigned=>unsigned
     ((let ((y (constantize y)))
-       (cond ((and (register-p r) (eql y -1)) ; special-case "OR reg, all-ones"
+       (cond ((and (gpr-tn-p r) (eql y -1)) ; special-case "OR reg, all-ones"
               ;; I have yet to elicit this case. Can it happen?
               (inst mov r -1))
              (t
@@ -278,7 +298,7 @@
   (:generator 2
     (cond ((and (sc-is x any-reg) (sc-is y any-reg) (sc-is r any-reg)
                 (not (location= x r)))
-           (inst lea r (make-ea :qword :base x :index y)))
+           (inst lea r (ea x y)))
           (t
            (move r x)
            (inst add r y)))))
@@ -295,7 +315,7 @@
     (let ((y (fixnumize y)))
       (cond ((and (not (location= x r))
                   (typep y '(signed-byte 32)))
-             (inst lea r (make-ea :qword :base x :disp y)))
+             (inst lea r (ea y x)))
             (t
              (move r x)
              (inst add r (constantize y)))))))
@@ -318,7 +338,7 @@
   (:generator 5
     (cond ((and (sc-is x signed-reg) (sc-is y signed-reg) (sc-is r signed-reg)
                 (not (location= x r)))
-           (inst lea r (make-ea :qword :base x :index y)))
+           (inst lea r (ea x y)))
           (t
            (move r x)
            (inst add r y)))))
@@ -370,7 +390,7 @@
     (cond ((and (sc-is x signed-reg) (sc-is r signed-reg)
                 (not (location= x r))
                 (typep y '(signed-byte 32)))
-           (inst lea r (make-ea :qword :base x :disp y)))
+           (inst lea r (ea y x)))
           (t
            (move r x)
            (cond ((= y 1)
@@ -397,7 +417,7 @@
   (:generator 5
     (cond ((and (sc-is x unsigned-reg) (sc-is y unsigned-reg)
                 (sc-is r unsigned-reg) (not (location= x r)))
-           (inst lea r (make-ea :qword :base x :index y)))
+           (inst lea r (ea x y)))
           (t
            (move r x)
            (inst add r y)))))
@@ -418,7 +438,7 @@
     (cond ((and (sc-is x unsigned-reg) (sc-is r unsigned-reg)
                 (not (location= x r))
                 (typep y '(unsigned-byte 31)))
-           (inst lea r (make-ea :qword :base x :disp y)))
+           (inst lea r (ea y x)))
           (t
            (move r x)
            (cond ((= y 1)
@@ -559,9 +579,8 @@
     (if (location= quo eax)
         (inst shl eax n-fixnum-tag-bits)
         (if (= n-fixnum-tag-bits 1)
-            (inst lea quo (make-ea :qword :base eax :index eax))
-            (inst lea quo (make-ea :qword :index eax
-                                   :scale (ash 1 n-fixnum-tag-bits)))))
+            (inst lea quo (ea eax eax))
+            (inst lea quo (ea nil eax (ash 1 n-fixnum-tag-bits)))))
     (move rem edx)))
 
 (define-vop (fast-truncate-c/fixnum=>fixnum fast-safe-arith-op)
@@ -588,9 +607,8 @@
     (if (location= quo eax)
         (inst shl eax n-fixnum-tag-bits)
         (if (= n-fixnum-tag-bits 1)
-            (inst lea quo (make-ea :qword :base eax :index eax))
-            (inst lea quo (make-ea :qword :index eax
-                                   :scale (ash 1 n-fixnum-tag-bits)))))
+            (inst lea quo (ea eax eax))
+            (inst lea quo (ea nil eax (ash 1 n-fixnum-tag-bits)))))
     (move rem edx)))
 
 (define-vop (fast-truncate/unsigned=>unsigned fast-safe-arith-op)
@@ -717,11 +735,11 @@
   (:variant-vars modularp)
   (:generator 2
     (cond ((and (= amount 1) (not (location= number result)))
-           (inst lea result (make-ea :qword :base number :index number)))
+           (inst lea result (ea number number)))
           ((and (= amount 2) (not (location= number result)))
-           (inst lea result (make-ea :qword :index number :scale 4)))
+           (inst lea result (ea nil number 4)))
           ((and (= amount 3) (not (location= number result)))
-           (inst lea result (make-ea :qword :index number :scale 8)))
+           (inst lea result (ea nil number 8)))
           (t
            (move result number)
            (cond ((< -64 amount 64)
@@ -783,11 +801,11 @@ constant shift greater than word length")))
   (:note "inline ASH")
   (:generator 3
     (cond ((and (= amount 1) (not (location= number result)))
-           (inst lea result (make-ea :qword :base number :index number)))
+           (inst lea result (ea number number)))
           ((and (= amount 2) (not (location= number result)))
-           (inst lea result (make-ea :qword :index number :scale 4)))
+           (inst lea result (ea nil number 4)))
           ((and (= amount 3) (not (location= number result)))
-           (inst lea result (make-ea :qword :index number :scale 8)))
+           (inst lea result (ea nil number 8)))
           (t
            (move result number)
            (cond ((plusp amount) (inst shl result amount))
@@ -810,11 +828,11 @@ constant shift greater than word length")))
   (:note "inline ASH")
   (:generator 3
     (cond ((and (= amount 1) (not (location= number result)))
-           (inst lea result (make-ea :qword :base number :index number)))
+           (inst lea result (ea number number)))
           ((and (= amount 2) (not (location= number result)))
-           (inst lea result (make-ea :qword :index number :scale 4)))
+           (inst lea result (ea nil number 4)))
           ((and (= amount 3) (not (location= number result)))
-           (inst lea result (make-ea :qword :index number :scale 8)))
+           (inst lea result (ea nil number 8)))
           (t
            (move result number)
            (cond ((< -64 amount 64) ;; XXXX
@@ -971,16 +989,17 @@ constant shift greater than word length")))
     (inst sar result :cl)
     (inst and result (lognot fixnum-tag-mask))))
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (defknown %lea (integer integer (member 1 2 4 8 16) (signed-byte 64))
   integer
   (foldable flushable movable))
 
+;;; FIXME: arg order should be (DISP BASE INDEX SCALE) to match EA constructor
 (defun %lea (base index scale disp)
   (+ base (* index scale) disp))
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (define-vop (%lea/unsigned=>unsigned)
   (:translate %lea)
@@ -994,8 +1013,7 @@ constant shift greater than word length")))
   (:results (r :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (inst lea r (make-ea :qword :base base :index index
-                         :scale scale :disp disp))))
+    (inst lea r (ea disp base index scale))))
 
 (define-vop (%lea/signed=>signed)
   (:translate %lea)
@@ -1009,8 +1027,7 @@ constant shift greater than word length")))
   (:results (r :scs (signed-reg)))
   (:result-types signed-num)
   (:generator 4
-    (inst lea r (make-ea :qword :base base :index index
-                         :scale scale :disp disp))))
+    (inst lea r (ea disp base index scale))))
 
 (define-vop (%lea/fixnum=>fixnum)
   (:translate %lea)
@@ -1024,8 +1041,7 @@ constant shift greater than word length")))
   (:results (r :scs (any-reg)))
   (:result-types tagged-num)
   (:generator 3
-    (inst lea r (make-ea :qword :base base :index index
-                         :scale scale :disp disp))))
+    (inst lea r (ea disp base index scale))))
 
 ;;; FIXME: before making knowledge of this too public, it needs to be
 ;;; fixed so that it's actually _faster_ than the non-CMOV version; at
@@ -1219,17 +1235,17 @@ constant shift greater than word length")))
                    (return-from ensure-not-mem+mem (values x it)))
                   ((typep y '(unsigned-byte 32))
                    ;; Rather than a RIP-relative constant, load a dword (w/o sign-extend)
-                   (inst mov (reg-in-size temp-reg-tn :dword) y)
+                   (inst mov :dword temp-reg-tn y)
                    (return-from ensure-not-mem+mem (values x temp-reg-tn))))
            (setq y (register-inline-constant :qword y)))
-         (cond ((or (gpr-p x) (gpr-p y))
+         (cond ((or (gpr-tn-p x) (gpr-tn-p y))
                 (values x y))
                (t
                 (inst mov temp-reg-tn x)
                 (values temp-reg-tn y))))))
 
 (defun immediate-operand-smallest-nbits (x)
-  (declare (word x))
+  (declare (type word x))
   (typecase x
     ((unsigned-byte  8)  8)
     ((unsigned-byte 16) 16)
@@ -1270,24 +1286,21 @@ constant shift greater than word length")))
     (cond ((not size)
            ;; Ensure that both operands are acceptable
            ;; by possibly loading one into TEMP-REG-TN
-           (multiple-value-setq (x y) (ensure-not-mem+mem x y)))
+           (multiple-value-setq (x y) (ensure-not-mem+mem x y))
+           (inst test :qword x y))
           ((sc-is x control-stack unsigned-stack signed-stack)
            ;; Otherwise, when using an immediate operand smaller
            ;; than 64 bits, narrow the reg/mem operand to match.
            (let ((disp (frame-byte-offset (tn-offset x))))
              (when reducible-to-byte-p
                (setq size :byte disp (1+ disp) y (ash y -8)))
-             (setq x (make-ea size :base rbp-tn :disp disp))))
+             (inst test size (ea disp rbp-tn) y)))
           (t
-           (aver (gpr-p x))
-           (if (and reducible-to-byte-p (<= (tn-offset x) 6)) ; 0, 2, 4, 6
-               ;; Use upper byte of word reg (AX -> AH, BX -> BX ...)
-               (setq x (make-random-tn :kind :normal
-                                       :sc (sc-or-lose 'byte-reg)
-                                       :offset (1+ (tn-offset x)))
-                     y (ash y -8))
-               (setq x (reg-in-size x size))))))
-  (inst test x y))
+           (aver (gpr-tn-p x))
+           (if (and reducible-to-byte-p (<= (tn-offset x) rbx-offset))
+               ;; Use upper byte of word reg {A,C,D,B}X -> {A,C,D,B}H
+               (inst test :byte `(,x . :high-byte) (ash y -8))
+               (inst test size x y))))))
 
 ;; Stolen liberally from the x86 32-bit implementation.
 (macrolet ((define-logtest-vops ()
@@ -1341,7 +1354,7 @@ constant shift greater than word length")))
   (:arg-types tagged-num (:constant (integer 0 #.(- 63 n-fixnum-tag-bits))))
   (:generator 4
     (let ((bit (+ y n-fixnum-tag-bits)))
-      (inst bt (if (<= bit 31) (reg-in-size x :dword) x) bit))))
+      (inst bt (if (<= bit 31) :dword :qword) x bit))))
 
 (define-vop (fast-logbitp/signed fast-conditional/signed)
   (:args (x :scs (signed-reg signed-stack))
@@ -1374,7 +1387,7 @@ constant shift greater than word length")))
     (inst bt x y)))
 
 (defun emit-optimized-cmp (x y)
-  (if (and (gpr-p x) (eql y 0))
+  (if (and (gpr-tn-p x) (eql y 0))
       ;; Amazingly (to me), use of TEST in lieu of CMP produces all the correct
       ;; flag bits for inequality comparison as well as EQL comparison.
       ;; You'd think that the Jxx instruction should examine _only_ the S flag,
@@ -1600,8 +1613,8 @@ constant shift greater than word length")))
              fast-ash-left/unsigned=>unsigned))
 (deftransform ash-left-mod64 ((integer count)
                               ((unsigned-byte 64) (unsigned-byte 6)))
-  (when (sb!c::constant-lvar-p count)
-    (sb!c::give-up-ir1-transform))
+  (when (sb-c::constant-lvar-p count)
+    (sb-c::give-up-ir1-transform))
   '(%primitive fast-ash-left-mod64/unsigned=>unsigned integer count))
 
 (define-vop (fast-ash-left-modfx-c/fixnum=>fixnum
@@ -1612,16 +1625,16 @@ constant shift greater than word length")))
              fast-ash-left/fixnum=>fixnum))
 (deftransform ash-left-modfx ((integer count)
                               (fixnum (unsigned-byte 6)))
-  (when (sb!c::constant-lvar-p count)
-    (sb!c::give-up-ir1-transform))
+  (when (sb-c::constant-lvar-p count)
+    (sb-c::give-up-ir1-transform))
   '(%primitive fast-ash-left-modfx/fixnum=>fixnum integer count))
 
-(in-package "SB!C")
+(in-package "SB-C")
 
-(defknown sb!vm::%lea-mod64 (integer integer (member 1 2 4 8) (signed-byte 64))
+(defknown sb-vm::%lea-mod64 (integer integer (member 1 2 4 8) (signed-byte 64))
   (unsigned-byte 64)
   (foldable flushable movable))
-(defknown sb!vm::%lea-modfx (integer integer (member 1 2 4 8) (signed-byte 64))
+(defknown sb-vm::%lea-modfx (integer integer (member 1 2 4 8) (signed-byte 64))
   fixnum
   (foldable flushable movable))
 
@@ -1631,39 +1644,39 @@ constant shift greater than word length")))
              (constant-lvar-p disp))
     (cut-to-width base :untagged width nil)
     (cut-to-width index :untagged width nil)
-    'sb!vm::%lea-mod64))
+    'sb-vm::%lea-mod64))
 (define-modular-fun-optimizer %lea ((base index scale disp) :tagged t :width width)
-  (when (and (<= width (- sb!vm:n-word-bits sb!vm:n-fixnum-tag-bits))
+  (when (and (<= width (- sb-vm:n-word-bits sb-vm:n-fixnum-tag-bits))
              (constant-lvar-p scale)
              (constant-lvar-p disp))
     (cut-to-width base :tagged width t)
     (cut-to-width index :tagged width t)
-    'sb!vm::%lea-modfx))
+    'sb-vm::%lea-modfx))
 
 #+sb-xc-host
 (progn
-  (defun sb!vm::%lea-mod64 (base index scale disp)
+  (defun sb-vm::%lea-mod64 (base index scale disp)
     (ldb (byte 64 0) (%lea base index scale disp)))
-  (defun sb!vm::%lea-modfx (base index scale disp)
-    (mask-signed-field (- sb!vm:n-word-bits sb!vm:n-fixnum-tag-bits)
+  (defun sb-vm::%lea-modfx (base index scale disp)
+    (mask-signed-field (- sb-vm:n-word-bits sb-vm:n-fixnum-tag-bits)
                        (%lea base index scale disp))))
 #-sb-xc-host
 (progn
-  (defun sb!vm::%lea-mod64 (base index scale disp)
+  (defun sb-vm::%lea-mod64 (base index scale disp)
     (let ((base (logand base #xffffffffffffffff))
           (index (logand index #xffffffffffffffff)))
       ;; can't use modular version of %LEA, as we only have VOPs for
       ;; constant SCALE and DISP.
       (ldb (byte 64 0) (+ base (* index scale) disp))))
-  (defun sb!vm::%lea-modfx (base index scale disp)
-    (let* ((fixnum-width (- sb!vm:n-word-bits sb!vm:n-fixnum-tag-bits))
+  (defun sb-vm::%lea-modfx (base index scale disp)
+    (let* ((fixnum-width (- sb-vm:n-word-bits sb-vm:n-fixnum-tag-bits))
            (base (mask-signed-field fixnum-width base))
            (index (mask-signed-field fixnum-width index)))
       ;; can't use modular version of %LEA, as we only have VOPs for
       ;; constant SCALE and DISP.
       (mask-signed-field fixnum-width (+ base (* index scale) disp)))))
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (define-vop (%lea-mod64/unsigned=>unsigned
              %lea/unsigned=>unsigned)
@@ -1711,23 +1724,23 @@ constant shift greater than word length")))
 ;;;; bignum stuff
 
 (define-vop (bignum-length get-header-data)
-  (:translate sb!bignum:%bignum-length)
+  (:translate sb-bignum:%bignum-length)
   (:policy :fast-safe))
 
 (define-vop (bignum-set-length set-header-data)
-  (:translate sb!bignum:%bignum-set-length)
+  (:translate sb-bignum:%bignum-set-length)
   (:policy :fast-safe))
 
 (define-full-reffer bignum-ref * bignum-digits-offset other-pointer-lowtag
-  (unsigned-reg) unsigned-num sb!bignum:%bignum-ref)
+  (unsigned-reg) unsigned-num sb-bignum:%bignum-ref)
 (define-full-reffer+offset bignum-ref-with-offset * bignum-digits-offset
   other-pointer-lowtag (unsigned-reg) unsigned-num
-  sb!bignum:%bignum-ref-with-offset)
+  sb-bignum:%bignum-ref-with-offset)
 (define-full-setter bignum-set * bignum-digits-offset other-pointer-lowtag
-  (unsigned-reg) unsigned-num sb!bignum:%bignum-set)
+  (unsigned-reg) unsigned-num sb-bignum:%bignum-set)
 
 (define-vop (digit-0-or-plus)
-  (:translate sb!bignum:%digit-0-or-plusp)
+  (:translate sb-bignum:%digit-0-or-plusp)
   (:policy :fast-safe)
   (:args (digit :scs (unsigned-reg)))
   (:arg-types unsigned-num)
@@ -1741,7 +1754,7 @@ constant shift greater than word length")))
 ;;; 8. This is easy to deal with and may save a fixnum-word
 ;;; conversion.
 (define-vop (add-w/carry)
-  (:translate sb!bignum:%add-with-carry)
+  (:translate sb-bignum:%add-with-carry)
   (:policy :fast-safe)
   (:args (a :scs (unsigned-reg) :target result)
          (b :scs (unsigned-reg unsigned-stack) :to :eval)
@@ -1762,7 +1775,7 @@ constant shift greater than word length")))
 ;;; Note: the borrow is 1 for no borrow and 0 for a borrow, the opposite
 ;;; of the x86-64 convention.
 (define-vop (sub-w/borrow)
-  (:translate sb!bignum:%subtract-with-borrow)
+  (:translate sb-bignum:%subtract-with-borrow)
   (:policy :fast-safe)
   (:args (a :scs (unsigned-reg) :to :eval :target result)
          (b :scs (unsigned-reg unsigned-stack) :to :result)
@@ -1780,7 +1793,7 @@ constant shift greater than word length")))
 
 
 (define-vop (bignum-mult-and-add-3-arg)
-  (:translate sb!bignum:%multiply-and-add)
+  (:translate sb-bignum:%multiply-and-add)
   (:policy :fast-safe)
   (:args (x :scs (unsigned-reg) :target eax)
          (y :scs (unsigned-reg unsigned-stack))
@@ -1802,7 +1815,7 @@ constant shift greater than word length")))
     (move lo eax)))
 
 (define-vop (bignum-mult-and-add-4-arg)
-  (:translate sb!bignum:%multiply-and-add)
+  (:translate sb-bignum:%multiply-and-add)
   (:policy :fast-safe)
   (:args (x :scs (unsigned-reg) :target eax)
          (y :scs (unsigned-reg unsigned-stack))
@@ -1828,7 +1841,7 @@ constant shift greater than word length")))
 
 
 (define-vop (bignum-mult)
-  (:translate sb!bignum:%multiply)
+  (:translate sb-bignum:%multiply)
   (:policy :fast-safe)
   (:args (x :scs (unsigned-reg) :target eax)
          (y :scs (unsigned-reg unsigned-stack)))
@@ -1881,10 +1894,10 @@ constant shift greater than word length")))
     (inst and hi (lognot fixnum-tag-mask))))
 
 (define-vop (bignum-lognot lognot-mod64/unsigned=>unsigned)
-  (:translate sb!bignum:%lognot))
+  (:translate sb-bignum:%lognot))
 
 (define-vop (fixnum-to-digit)
-  (:translate sb!bignum:%fixnum-to-digit)
+  (:translate sb-bignum:%fixnum-to-digit)
   (:policy :fast-safe)
   (:args (fixnum :scs (any-reg control-stack) :target digit))
   (:arg-types tagged-num)
@@ -1898,7 +1911,7 @@ constant shift greater than word length")))
     (inst sar digit n-fixnum-tag-bits)))
 
 (define-vop (bignum-floor)
-  (:translate sb!bignum:%bigfloor)
+  (:translate sb-bignum:%bigfloor)
   (:policy :fast-safe)
   (:args (div-high :scs (unsigned-reg) :target edx)
          (div-low :scs (unsigned-reg) :target eax)
@@ -1919,7 +1932,7 @@ constant shift greater than word length")))
     (move rem edx)))
 
 (define-vop (signify-digit)
-  (:translate sb!bignum:%fixnum-digit-with-correct-sign)
+  (:translate sb-bignum:%fixnum-digit-with-correct-sign)
   (:policy :fast-safe)
   (:args (digit :scs (unsigned-reg unsigned-stack) :target res))
   (:arg-types unsigned-num)
@@ -1934,7 +1947,7 @@ constant shift greater than word length")))
       (inst shl res n-fixnum-tag-bits))))
 
 (define-vop (digit-ashr)
-  (:translate sb!bignum:%ashr)
+  (:translate sb-bignum:%ashr)
   (:policy :fast-safe)
   (:args (digit :scs (unsigned-reg unsigned-stack) :target result)
          (count :scs (unsigned-reg) :target ecx))
@@ -1950,7 +1963,7 @@ constant shift greater than word length")))
     (inst sar result :cl)))
 
 (define-vop (digit-ashr/c)
-  (:translate sb!bignum:%ashr)
+  (:translate sb-bignum:%ashr)
   (:policy :fast-safe)
   (:args (digit :scs (unsigned-reg unsigned-stack) :target result))
   (:arg-types unsigned-num (:constant (integer 0 63)))
@@ -1964,14 +1977,14 @@ constant shift greater than word length")))
     (inst sar result count)))
 
 (define-vop (digit-lshr digit-ashr)
-  (:translate sb!bignum:%digit-logical-shift-right)
+  (:translate sb-bignum:%digit-logical-shift-right)
   (:generator 1
     (move result digit)
     (move ecx count)
     (inst shr result :cl)))
 
 (define-vop (digit-ashl digit-ashr)
-  (:translate sb!bignum:%ashl)
+  (:translate sb-bignum:%ashl)
   (:generator 1
     (move result digit)
     (move ecx count)
@@ -1999,54 +2012,50 @@ constant shift greater than word length")))
                                                  other-pointer-lowtag))))))
 
 ;; Specialised mask-signed-field VOPs.
-(define-vop (mask-signed-field-word/c)
-  (:translate sb!c::mask-signed-field)
-  (:policy :fast-safe)
-  (:args (x :scs (signed-reg unsigned-reg) :target r))
-  (:arg-types (:constant (integer 0 64)) untagged-num)
-  (:results (r :scs (signed-reg)))
-  (:result-types signed-num)
-  (:info width)
-  (:generator 3
-    (cond ((zerop width)
-           (zeroize r))
-          ((= width 64)
-           (move r x))
-          ((member width '(32 16 8))
-           (inst movsx r (reg-in-size x (bits->size width))))
-          (t
-           (move r x)
-           (let ((delta (- n-word-bits width)))
-             (inst shl r delta)
-             (inst sar r delta))))))
+(flet ((shift-unshift (reg width)
+         (let ((shift (- n-word-bits width)))
+           ;; Shift of 64 is effectively a shift of 0 due to masking by the CPU.
+           ;; It can't happen, because size = 0 was dealt with in IR1
+           (aver (/= shift 64))
+           (unless (= shift 0)
+             (inst shl reg shift)
+             (inst sar reg shift)))))
+ (define-vop (mask-signed-field-word/c)
+   (:translate sb-c::mask-signed-field)
+   (:policy :fast-safe)
+   (:args (x :scs (signed-reg unsigned-reg) :target r))
+   (:arg-types (:constant (integer 0 64)) untagged-num)
+   (:results (r :scs (signed-reg)))
+   (:result-types signed-num)
+   (:info width)
+   (:generator 3
+     (case width
+       ((8 16 32)
+        (inst movsx `(,(bits->size width) :qword) r x))
+       (t
+        (move r x)
+        (shift-unshift r width)))))
 
-(define-vop (mask-signed-field-bignum/c)
-  (:translate sb!c::mask-signed-field)
-  (:policy :fast-safe)
-  (:args (x :scs (descriptor-reg) :target r))
-  (:arg-types (:constant (integer 0 64)) bignum)
-  (:results (r :scs (signed-reg)))
-  (:result-types signed-num)
-  (:info width)
-  (:generator 4
-    (cond ((zerop width)
-           (zeroize r))
-          ((member width '(8 16 32 64))
-           (ecase width
-             (64 (loadw r x bignum-digits-offset other-pointer-lowtag))
-             ((32 16 8)
-              (inst movsx r (make-ea (bits->size width)
-                                     :base x
-                                     :disp (- (* bignum-digits-offset n-word-bytes)
-                                              other-pointer-lowtag))))))
-          (t
-           (loadw r x bignum-digits-offset other-pointer-lowtag)
-           (let ((delta (- n-word-bits width)))
-             (inst shl r delta)
-             (inst sar r delta))))))
+ (define-vop (mask-signed-field-bignum/c)
+   (:translate sb-c::mask-signed-field)
+   (:policy :fast-safe)
+   (:args (x :scs (descriptor-reg) :target r))
+   (:arg-types (:constant (integer 0 64)) bignum)
+   (:results (r :scs (signed-reg)))
+   (:result-types signed-num)
+   (:info width)
+   (:generator 4
+     (case width
+       ((8 16 32)
+        (inst movsx `(,(bits->size width) :qword)
+              r
+              (ea (- (* bignum-digits-offset n-word-bytes) other-pointer-lowtag) x)))
+       (t
+        (loadw r x bignum-digits-offset other-pointer-lowtag)
+        (shift-unshift r width))))))
 
 (define-vop (mask-signed-field-fixnum)
-  (:translate sb!c::mask-signed-field)
+  (:translate sb-c::mask-signed-field)
   (:policy :fast-safe)
   (:args (x :scs (descriptor-reg) :target r))
   (:arg-types (:constant (eql #.n-fixnum-bits)) t)
@@ -2091,7 +2100,7 @@ constant shift greater than word length")))
           (inst and r (or (plausible-signed-imm32-operand-p mask)
                           (constantize mask))))))))
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (defun *-transformer (y node)
   (cond
@@ -2120,7 +2129,7 @@ constant shift greater than word length")))
                  :node node)
   "recode as leas, shifts and adds"
   (*-transformer (lvar-value y) node))
-(deftransform sb!vm::*-mod64
+(deftransform sb-vm::*-mod64
     ((x y) ((unsigned-byte 64) (constant-arg (unsigned-byte 64)))
      (unsigned-byte 64)
      :important nil
@@ -2135,7 +2144,7 @@ constant shift greater than word length")))
                  :node node)
   "recode as leas, shifts and adds"
   (*-transformer (lvar-value y) node))
-(deftransform sb!vm::*-modfx
+(deftransform sb-vm::*-modfx
     ((x y) (fixnum (constant-arg (unsigned-byte 64)))
      fixnum
      :important nil
