@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; type frobbing VOPs
 
@@ -63,6 +63,16 @@
     (inst movzx result (make-ea :byte :base function
                                       :disp (- fun-pointer-lowtag)))))
 
+(define-vop (fun-header-data)
+  (:translate fun-header-data)
+  (:policy :fast-safe)
+  (:args (x :scs (descriptor-reg)))
+  (:results (res :scs (unsigned-reg)))
+  (:result-types positive-fixnum)
+  (:generator 6
+    (loadw res x 0 fun-pointer-lowtag)
+    (inst shr res n-widetag-bits)))
+
 (define-vop (get-header-data)
   (:translate get-header-data)
   (:policy :fast-safe)
@@ -73,17 +83,6 @@
     (loadw res x 0 other-pointer-lowtag)
     (inst shr res n-widetag-bits)))
 
-(define-vop (get-closure-length)
-  (:translate get-closure-length)
-  (:policy :fast-safe)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (unsigned-reg)))
-  (:result-types positive-fixnum)
-  (:generator 6
-    (loadw res x 0 fun-pointer-lowtag)
-    (inst shr res n-widetag-bits)
-    (inst and res short-header-max-words)))
-
 (define-vop (set-header-data)
   (:translate set-header-data)
   (:policy :fast-safe)
@@ -139,25 +138,52 @@
   (:results (sap :scs (sap-reg) :from (:argument 0)))
   (:result-types system-area-pointer)
   (:generator 10
-    (loadw sap code 0 other-pointer-lowtag)
-    (inst shl sap 2) ; shift out the two GC-reserved bits
-    (inst shr sap (+ 2 n-widetag-bits))
-    (inst lea sap (make-ea :byte :base code :index sap :scale 4
+    ;; load boxed header size in bytes
+    (loadw sap code 1 other-pointer-lowtag)
+    (inst lea sap (make-ea :byte :base code :index sap
                            :disp (- other-pointer-lowtag)))))
 
+(eval-when (:compile-toplevel)
+  (aver (not (logtest code-header-widetag #b11000000))))
+
+(define-vop (code-trailer-ref)
+  (:translate code-trailer-ref)
+  (:policy :fast-safe)
+  (:args (code :scs (descriptor-reg) :to (:result 0))
+         (offset :scs (signed-reg immediate) :to (:result 0)))
+  (:arg-types * fixnum)
+  (:results (res :scs (unsigned-reg) :from (:argument 0)))
+  (:result-types unsigned-num)
+  (:generator 10
+    ;; get the object size in words
+    (inst mov res (make-ea :dword :base code :disp (- other-pointer-lowtag)))
+    (inst shl res 2) ; shift out the GC bits
+    ;; Then shift right to clear the widetag, plus 2 more to the right since we just
+    ;; left-shifted to zeroize bits. Then shift left 2 to convert words to bytes.
+    ;; The '>>2' and '<<2' cancel out because we don't need to clear all 8 bits
+    ;; of the widetag, as CODE-HEADER-WIDETAG already has bits 6 and 7 clear.
+    ;; Other places assume the same, though without much commentary.
+    ;; It's brittle magic, save for the AVER above which ensures that it works.
+    (inst shr res n-widetag-bits)
+    (cond ((sc-is offset immediate)
+           (inst mov res (make-ea :dword :disp (- (tn-value offset) other-pointer-lowtag)
+                                  :base code :index res)))
+          (t
+           (inst add res offset)
+           (inst mov res (make-ea :dword :base code :index res
+                                  :disp (- other-pointer-lowtag)))))))
+
 (define-vop (compute-fun)
   (:args (code :scs (descriptor-reg) :to (:result 0))
-         (offset :scs (signed-reg unsigned-reg) :to (:result 0)))
+         (offset :scs (signed-reg unsigned-reg) :to :eval :target func))
   (:arg-types * positive-fixnum)
-  (:results (func :scs (descriptor-reg) :from (:argument 0)))
-  (:generator 10
-    (loadw func code 0 other-pointer-lowtag)
-    (inst shl func 2) ; shift out the two GC-reserved bits
-    (inst shr func (+ 2 n-widetag-bits))
-    (inst lea func
-          (make-ea :byte :base offset :index func :scale 4
-                   :disp (- fun-pointer-lowtag other-pointer-lowtag)))
-    (inst add func code)))
+  (:results (func :scs (descriptor-reg) :from :eval))
+  (:generator 3
+    (move func offset)
+    ;; add boxed header size in bytes
+    (inst add func (make-ea :dword :base code :disp (- 4 other-pointer-lowtag)))
+    (inst lea func (make-ea :byte :base code :index func
+                            :disp (- fun-pointer-lowtag other-pointer-lowtag)))))
 
 ;;; This vop is quite magical - because 'closure-fun' is a raw program counter,
 ;;; as soon as it's loaded into a register, it prevents the underlying fun from
@@ -212,10 +238,10 @@
 
 ;;;; other miscellaneous VOPs
 
-(defknown sb!unix::receive-pending-interrupt () (values))
-(define-vop (sb!unix::receive-pending-interrupt)
+(defknown sb-unix::receive-pending-interrupt () (values))
+(define-vop (sb-unix::receive-pending-interrupt)
   (:policy :fast-safe)
-  (:translate sb!unix::receive-pending-interrupt)
+  (:translate sb-unix::receive-pending-interrupt)
   (:generator 1
     (inst break pending-interrupt-trap)))
 
